package com.google.android.demo;

import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Bundle;

import com.google.android.maps.MapActivity;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayController;
import com.google.android.maps.Point;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.GetMethod;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

public class GOTV extends MapActivity {
    private LocationManager locManager;

    private Point getMyLocation() {
        List<LocationProvider> provs = locManager.getProviders();
        Location loc = locManager.getCurrentLocation(provs.get(0).getName());
        return new Point((int) (loc.getLatitude() * 1e6), (int) (loc
                .getLongitude() * 1e6));
    }

    private Point[] getPizzaPlaces(Point origin) {
        HttpClient client = new HttpClient();
        GetMethod yelp = new GetMethod(
                "http://api.yelp.com/business_review_search");
        NameValuePair[] params = new NameValuePair[5];
        params[0] = new NameValuePair("term", "pizza");
        params[1] = new NameValuePair("lat", "" + origin.getLatitudeE6()
                / (float) 1e6);
        params[2] = new NameValuePair("long", "" + origin.getLongitudeE6()
                / (float) 1e6);
        params[3] = new NameValuePair("radius", "5");
        params[4] = new NameValuePair("ywsid", "<INSERT YELP API KEY HERE>");
        yelp.setQueryString(params);
        try {
            client.executeMethod(yelp);
            String body = yelp.getResponseBodyAsString();
            JSONObject jso = new JSONObject(body);
            JSONArray bizzes = jso.getJSONArray("businesses");
            Point[] points = new Point[bizzes.length()];
            for (int i = 0; i < bizzes.length(); ++i) {
                JSONObject biz = bizzes.getJSONObject(i);
                int lat = (int) (Double.parseDouble(biz.getString("latitude")) * 1e6);
                int lng = (int) (Double.parseDouble(biz.getString("longitude")) * 1e6);
                points[i] = new Point(lat, lng);
            }
            return points;
        } catch (HttpException e) {
            return null;
        } catch (IOException e) {
            return null;
        } catch (JSONException e) {
            return null;
        }
    }

    private Point[] getAddresses(Point origin) {
        double lat = origin.getLatitudeE6() / (double) 1e6;
        double lon = origin.getLongitudeE6() / (double) 1e6;
        InputStream is;
        try {
            is = openFileInput("address_list");
        } catch (FileNotFoundException e1) {
            return null;
        }
        StringBuffer sb = new StringBuffer(1024);
        byte[] bytes = new byte[1024];
        try {
            while (is.read(bytes) > -1) {
                sb.append(new String(bytes));
            }
        } catch (IOException e) {
            return null;
        }
        String[] lines = sb.toString().split("\n");
        ArrayList<Point> points = new ArrayList<Point>(lines.length);
        for (String line : lines) {
            Geocoder coder = new Geocoder();
            try {
                Address addr = (coder.getFromLocationName(line.trim(), lat,
                        lon, 2, 2))[0];
                points.add(new Point((int) (addr.getLatitude() * 1e6),
                        (int) (addr.getLongitude() * 1e6)));
            } catch (Exception e) {
                continue;
            }
        }
        return points.toArray(new Point[points.size()]);
    }

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        locManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        setContentView(R.layout.map);
    }

    @Override
    protected void onStart() {
        super.onStart();
        MapView mv = (MapView) findViewById(R.id.map);
        OverlayController ocon = mv.createOverlayController();

        Point[] myLoc = new Point[] {getMyLocation()};
        Overlay myLocOverlay = new PointOverlay(myLoc, this.getResources()
                .getDrawable(R.drawable.blue_pin));
        ocon.add(myLocOverlay, true);

        Point[] addrs = getAddresses(myLoc[0]);
        Overlay addrOverlay = null;
        addrOverlay = new PointOverlay(addrs, this.getResources().getDrawable(
                R.drawable.red_pin));
        ocon.add(addrOverlay, true);

        Point[] pizza = getPizzaPlaces(myLoc[0]);
        Overlay pizzaOverlay = new PointOverlay(pizza, this.getResources()
                .getDrawable(R.drawable.green_pin));
        ocon.add(pizzaOverlay, true);

        ocon.activate(myLocOverlay, false);
        ocon.activate(addrOverlay, false);
        ocon.activate(pizzaOverlay, false);

        mv.getController().animateTo(myLoc[0]);
    }
}
